import 'dart:async';

import 'package:animated_widgets/widgets/rotation_animated.dart';
import 'package:animated_widgets/widgets/shake_animated_widget.dart';
import 'package:mmoo_base_recharge/card/utils/common_utils.dart';
import 'package:mmoo_base_recharge/card/utils/media_util.dart';
import 'package:mmoo_base_recharge/generated/l10n.dart';
import 'package:mmoo_base_recharge/card/entity/card_state_entity.dart';
import 'package:mmoo_base_recharge/card/utils/recharge_constant.dart';
import 'package:mmoo_base_recharge/card/utils/keyboard_height_widget.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:mmoo_base/entity/card_entity.dart';
import 'package:mmoo_base/ui/button_widget.dart';
import 'package:mmoo_base/ui/edit_text_widget.dart';
import 'package:mmoo_base/utils/code_utils.dart';
import 'package:mmoo_base/utils/navigator_utils.dart';
import 'package:mmoo_country_pickers/country.dart';
import 'package:mmoo_country_pickers/country_picker_dialog.dart';
import 'package:mmoo_country_pickers/utils/utils.dart';

//address
class AddressPage extends StatefulWidget {
  final CardEntity cardInfo;
  final logEvent;

  const AddressPage(this.cardInfo, {Key? key, this.logEvent}) : super(key: key);

  @override
  _AddressPageState createState() => _AddressPageState();
}

class _AddressPageState extends State<AddressPage> {
  
   int TEXT_SIZE_SIGNATURE = 40;
   int TEXT_NUMBER_NAME = 30;
   int TEXT_SIZE_PASS = 50;
   int TEXT_SIZE_ACCOUNT = 100;

  late CardEntity cardInfo;
  Timer? timer;
  bool cardAnim = false;
  bool cardCheck = false;
  bool cardCheckEmail = false;

  FocusNode focusNode = FocusNode();
  FocusNode focusNode1 = FocusNode();
  FocusNode focusNode2 = FocusNode();
  FocusNode focusNode3 = FocusNode();
  FocusNode focusNode4 = FocusNode();
  FocusNode focusNode5 = FocusNode();
  FocusNode focusNode6 = FocusNode();
  FocusNode focusNode7 = FocusNode();
  FocusNode focusNode8 = FocusNode();

  List<CardStateList> usList = [];
  List<CardStateList> caList = [];
  int stateIndex = -1;

  @override
  void initState() {
    super.initState();
    cardInfo = widget.cardInfo;
    loadString();
  }

  void loadString() async {
    String json = await rootBundle.loadString("assets/card_state.json");
    var mapData = await compute(CodeUtils.decodeListResult, json);
    mapData.forEach((element) {
      CardStateEntity entity = CardStateEntity.fromJson(element);
      if (entity.state == 'US') {
        usList = entity.list ?? [];
      } else if (entity.state == 'CA') {
        caList = entity.list ?? [];
      }
    });
    if (cardInfo.billCountry == 'US') {
      stateIndex =
          usList.indexWhere((element) => element.name == cardInfo.stateName);
    } else if (cardInfo.billCountry == 'CA') {
      stateIndex =
          caList.indexWhere((element) => element.name == cardInfo.stateName);
    }
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      behavior: HitTestBehavior.opaque,
      onTap: () {
        focusNode.unfocus();
        focusNode1.unfocus();
        focusNode2.unfocus();
        focusNode3.unfocus();
        focusNode4.unfocus();
        focusNode5.unfocus();
        focusNode6.unfocus();
        focusNode7.unfocus();
        focusNode8.unfocus();
      },
      child: Scaffold(
        resizeToAvoidBottomInset: false,
        backgroundColor: Colors.white,
        body: SafeArea(
          maintainBottomViewPadding: true,
          child: Column(
            children: <Widget>[
              RechargeCommonUtils.getAppBarWidget(
                  context, S.of(context).strCardBilling,
                  black: true),
              Divider(
                  color: RechargeConstantColors.line_color,
                  height: ScreenUtil().setWidth(1)),
              Expanded(
                child: SingleChildScrollView(
                  physics: BouncingScrollPhysics(),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Padding(
                        padding: EdgeInsets.fromLTRB(
                            ScreenUtil().setWidth(15),
                            ScreenUtil().setWidth(24),
                            ScreenUtil().setWidth(15),
                            0),
                        child: Text.rich(
                          TextSpan(
                            text: '*',
                            style: RechargeConstantText.text14Blue,
                            children: <TextSpan>[
                              TextSpan(
                                  text: S.of(context).strCardCountry,
                                  style: RechargeConstantText.text14Hint2),
                            ],
                          ),
                        ),
                      ),
                      InkWell(
                        child: Padding(
                          padding: EdgeInsets.fromLTRB(
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(10),
                              ScreenUtil().setWidth(10),
                              ScreenUtil().setWidth(10)),
                          child: Row(
                            children: [
                              Expanded(
                                child: Text(
                                    cardInfo.countryName ??
                                        S.of(context).strCardCountryHint,
                                    style:
                                        cardInfo.countryName?.isEmpty == false
                                            ? RechargeConstantText.text20BlackBold
                                            : RechargeConstantText.text20HintBold),
                              ),
                              Image.asset(RechargeConstantIcons.card_right,
                                  width: ScreenUtil().setWidth(40),
                                  height: ScreenUtil().setWidth(40)),
                            ],
                          ),
                        ),
                        onTap: NavigatorUtils.debounce(() {
                          NavigatorUtils.showBaseDialog(
                              context: context,
                              name: 'CountryPickerDialog',
                              builder: (BuildContext showContext) {
                                return SafeArea(
                                    maintainBottomViewPadding: true,
                                    child: CountryPickerDialog(
                                        titlePadding: EdgeInsets.all(
                                            ScreenUtil().setWidth(8)),
                                        searchCursorColor:
                                            RechargeConstantColors.app_color,
                                        searchInputDecoration: InputDecoration(
                                            hintText: S
                                                .of(context)
                                                .strLoginAboutCountrySearch),
                                        isSearchable: true,
                                        title: Text(S
                                            .of(context)
                                            .strLoginAboutCountryChoose),
                                        onValuePicked: (country) {
                                          if (!mounted) return;
                                          cardInfo.countryName = country.name;
                                          cardInfo.phoneCode =
                                              country.phoneCode;
                                          if (cardInfo.billCountry !=
                                              country.isoCode) {
                                            stateIndex = -1;
                                            if (country.isoCode == 'US' ||
                                                country.isoCode == 'CA') {
                                              cardInfo.stateName = null;
                                            }
                                          }
                                          cardInfo.billCountry =
                                              country.isoCode;
                                          setState(() {});
                                        },
                                        itemBuilder: _buildDialogItem));
                              });
                        }),
                      ),
                      Visibility(
                        visible:
                            cardCheck && cardInfo.countryName?.isEmpty != false,
                        child: Padding(
                          padding: EdgeInsets.fromLTRB(
                              ScreenUtil().setWidth(15),
                              0,
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8)),
                          child: Text(S.of(context).strCardCountryTip,
                              style: RechargeConstantText.text12Red),
                        ),
                      ),
                      Container(
                          height: ScreenUtil().setWidth(10),
                          color: RechargeConstantColors.card_line_color),
                      Padding(
                        padding: EdgeInsets.fromLTRB(ScreenUtil().setWidth(15),
                            0, ScreenUtil().setWidth(15), 0),
                        child: Stack(
                          alignment: Alignment.center,
                          clipBehavior: Clip.none,
                          children: [
                            EditTextWidget(
                                keyboardType: ITextInputType.text,
                                maxLines: 1,
                                textStyle: RechargeConstantText.text16BlackBold,
                                inputText: cardInfo.billFirstName ?? '',
                                hintText: S.of(context).strCardFirst,
                                hintStyle: RechargeConstantText.text16Hint,
                                fieldCallBack: (content) {
                                  cardInfo.billFirstName = content;
                                  setState(() {});
                                },
                                contentPadding: EdgeInsets.fromLTRB(
                                    ScreenUtil().setWidth(6),
                                    ScreenUtil().setWidth(15),
                                    ScreenUtil().setWidth(6),
                                    ScreenUtil().setWidth(15)),
                                focusNode: focusNode,
                                cursorColor: RechargeConstantColors.app_color,
                                inputBorder: const UnderlineInputBorder(
                                    borderSide:
                                        BorderSide(color: Color(0xFFDDDDDD))),
                                focusedBorder: UnderlineInputBorder(
                                    borderSide: BorderSide(
                                        color: RechargeConstantColors.app_color)),
                                textInputAction: TextInputAction.next,
                                inputMaxLength:
                                    TEXT_SIZE_SIGNATURE),
                            Positioned(
                              left: 0,
                              child: Text('*', style: RechargeConstantText.text14Blue),
                            ),
                          ],
                        ),
                      ),
                      Visibility(
                        visible: cardCheck &&
                            (cardInfo.billFirstName?.length ?? 0) < 2,
                        child: Padding(
                          padding: EdgeInsets.fromLTRB(
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8),
                              ScreenUtil().setWidth(15),
                              0),
                          child: Text(S.of(context).strCardFirstTip,
                              style: RechargeConstantText.text12Red),
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.fromLTRB(ScreenUtil().setWidth(15),
                            0, ScreenUtil().setWidth(15), 0),
                        child: Stack(
                          alignment: Alignment.center,
                          clipBehavior: Clip.none,
                          children: [
                            EditTextWidget(
                                keyboardType: ITextInputType.text,
                                maxLines: 1,
                                textStyle: RechargeConstantText.text16BlackBold,
                                inputText: cardInfo.billLastName ?? '',
                                hintText: S.of(context).strCardLast,
                                hintStyle: RechargeConstantText.text16Hint,
                                fieldCallBack: (content) {
                                  cardInfo.billLastName = content;
                                  setState(() {});
                                },
                                contentPadding: EdgeInsets.fromLTRB(
                                    ScreenUtil().setWidth(6),
                                    ScreenUtil().setWidth(15),
                                    ScreenUtil().setWidth(6),
                                    ScreenUtil().setWidth(15)),
                                focusNode: focusNode1,
                                cursorColor: RechargeConstantColors.app_color,
                                inputBorder: const UnderlineInputBorder(
                                    borderSide:
                                        BorderSide(color: Color(0xFFDDDDDD))),
                                focusedBorder: UnderlineInputBorder(
                                    borderSide: BorderSide(
                                        color: RechargeConstantColors.app_color)),
                                textInputAction: TextInputAction.next,
                                inputMaxLength:
                                    TEXT_SIZE_SIGNATURE),
                            Positioned(
                              left: 0,
                              child: Text('*', style: RechargeConstantText.text14Blue),
                            ),
                          ],
                        ),
                      ),
                      Visibility(
                        visible: cardCheck &&
                            (cardInfo.billLastName?.length ?? 0) < 2,
                        child: Padding(
                          padding: EdgeInsets.fromLTRB(
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8),
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8)),
                          child: Text(S.of(context).strCardLastTip,
                              style: RechargeConstantText.text12Red),
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.fromLTRB(ScreenUtil().setWidth(15),
                            0, ScreenUtil().setWidth(15), 0),
                        child: Row(
                          children: [
                            Expanded(
                              child: Text(
                                  cardInfo.phoneCode?.isEmpty == false
                                      ? '${cardInfo.billCountry} +${cardInfo.phoneCode}'
                                      : '-- --',
                                  style: cardInfo.phoneCode?.isEmpty == false
                                      ? RechargeConstantText.text16BlackBold
                                      : RechargeConstantText.text16HintBold),
                            ),
                            Container(
                                color: RechargeConstantColors.line_color,
                                height: ScreenUtil().setWidth(20),
                                width: ScreenUtil().setWidth(1)),
                            SizedBox(width: ScreenUtil().setWidth(15)),
                            Expanded(
                              flex: 3,
                              child: Stack(
                                alignment: Alignment.center,
                                clipBehavior: Clip.none,
                                children: [
                                  EditTextWidget(
                                      keyboardType: ITextInputType.phone,
                                      maxLines: 1,
                                      textStyle: RechargeConstantText.text16BlackBold,
                                      inputText: cardInfo.phone ?? '',
                                      hintText: S.of(context).strCardPhone,
                                      hintStyle: RechargeConstantText.text16Hint,
                                      fieldCallBack: (content) {
                                        cardInfo.phone = content;
                                        setState(() {});
                                      },
                                      contentPadding: EdgeInsets.fromLTRB(
                                          ScreenUtil().setWidth(6),
                                          ScreenUtil().setWidth(15),
                                          ScreenUtil().setWidth(6),
                                          ScreenUtil().setWidth(15)),
                                      focusNode: focusNode2,
                                      cursorColor: RechargeConstantColors.app_color,
                                      inputBorder: const UnderlineInputBorder(
                                          borderSide: BorderSide(
                                              color: Color(0xFFDDDDDD))),
                                      focusedBorder: UnderlineInputBorder(
                                          borderSide: BorderSide(
                                              color: RechargeConstantColors.app_color)),
                                      textInputAction: TextInputAction.next,
                                      inputMaxLength:
                                          TEXT_NUMBER_NAME),
                                  Positioned(
                                    left: 0,
                                    child: Text('*',
                                        style: RechargeConstantText.text14Blue),
                                  ),
                                ],
                              ),
                            ),
                          ],
                        ),
                      ),
                      Visibility(
                        visible: cardCheck && (cardInfo.phone?.length ?? 0) < 5,
                        child: Padding(
                          padding: EdgeInsets.fromLTRB(
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8),
                              ScreenUtil().setWidth(15),
                              0),
                          child: Text(S.of(context).strCardPhoneTip,
                              style: RechargeConstantText.text12Red),
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.fromLTRB(ScreenUtil().setWidth(15),
                            0, ScreenUtil().setWidth(15), 0),
                        child: Stack(
                          alignment: Alignment.center,
                          clipBehavior: Clip.none,
                          children: [
                            EditTextWidget(
                                keyboardType: ITextInputType.text,
                                maxLines: 1,
                                textStyle: RechargeConstantText.text16BlackBold,
                                inputText: cardInfo.billEmail ?? '',
                                hintText: S.of(context).strCardEmail,
                                hintStyle: RechargeConstantText.text16Hint,
                                fieldCallBack: (content) {
                                  cardInfo.billEmail = content;
                                  cardCheckEmail = false;
                                  setState(() {});
                                },
                                contentPadding: EdgeInsets.fromLTRB(
                                    ScreenUtil().setWidth(6),
                                    ScreenUtil().setWidth(15),
                                    ScreenUtil().setWidth(6),
                                    ScreenUtil().setWidth(15)),
                                focusNode: focusNode8,
                                cursorColor: RechargeConstantColors.app_color,
                                inputBorder: const UnderlineInputBorder(
                                    borderSide:
                                        BorderSide(color: Color(0xFFDDDDDD))),
                                focusedBorder: UnderlineInputBorder(
                                    borderSide: BorderSide(
                                        color: RechargeConstantColors.app_color)),
                                textInputAction: TextInputAction.next,
                                inputMaxLength: TEXT_SIZE_PASS),
                            Positioned(
                              left: 0,
                              child: Text('*', style: RechargeConstantText.text14Blue),
                            ),
                          ],
                        ),
                      ),
                      Visibility(
                        visible: cardCheck &&
                            ((cardInfo.billEmail?.length ?? 0) < 2 ||
                                cardCheckEmail),
                        child: Padding(
                          padding: EdgeInsets.fromLTRB(
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8),
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8)),
                          child: Text(
                              cardCheckEmail
                                  ? S.of(context).strLoginSignEmailValid
                                  : S.of(context).strCardEmailTip,
                              style: RechargeConstantText.text12Red),
                        ),
                      ),
                      Container(
                          height: ScreenUtil().setWidth(10),
                          color: RechargeConstantColors.card_line_color),
                      Padding(
                        padding: EdgeInsets.fromLTRB(ScreenUtil().setWidth(15),
                            0, ScreenUtil().setWidth(15), 0),
                        child: Stack(
                          alignment: Alignment.center,
                          clipBehavior: Clip.none,
                          children: [
                            EditTextWidget(
                                keyboardType: ITextInputType.text,
                                maxLines: 1,
                                textStyle: RechargeConstantText.text16BlackBold,
                                inputText: cardInfo.billCity ?? '',
                                hintText: S.of(context).strCardCity,
                                hintStyle: RechargeConstantText.text16Hint,
                                fieldCallBack: (content) {
                                  cardInfo.billCity = content;
                                  setState(() {});
                                },
                                contentPadding: EdgeInsets.fromLTRB(
                                    ScreenUtil().setWidth(6),
                                    ScreenUtil().setWidth(15),
                                    ScreenUtil().setWidth(6),
                                    ScreenUtil().setWidth(15)),
                                focusNode: focusNode3,
                                cursorColor: RechargeConstantColors.app_color,
                                inputBorder: const UnderlineInputBorder(
                                    borderSide:
                                        BorderSide(color: Color(0xFFDDDDDD))),
                                focusedBorder: UnderlineInputBorder(
                                    borderSide: BorderSide(
                                        color: RechargeConstantColors.app_color)),
                                textInputAction: TextInputAction.next,
                                inputMaxLength: TEXT_SIZE_PASS),
                            Positioned(
                              left: 0,
                              child: Text('*', style: RechargeConstantText.text14Blue),
                            ),
                          ],
                        ),
                      ),
                      Visibility(
                        visible:
                            cardCheck && (cardInfo.billCity?.length ?? 0) < 2,
                        child: Padding(
                          padding: EdgeInsets.fromLTRB(
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8),
                              ScreenUtil().setWidth(15),
                              0),
                          child: Text(S.of(context).strCardCityTip,
                              style: RechargeConstantText.text12Red),
                        ),
                      ),
                      InkWell(
                        onTap: cardInfo.billCountry == 'US' ||
                                cardInfo.billCountry == 'CA'
                            ? NavigatorUtils.debounce(() {
                                showStateDialog(context);
                              })
                            : null,
                        child: Padding(
                          padding: EdgeInsets.fromLTRB(
                              ScreenUtil().setWidth(15),
                              0,
                              ScreenUtil().setWidth(15),
                              0),
                          child: Stack(
                            alignment: Alignment.center,
                            clipBehavior: Clip.none,
                            children: [
                              EditTextWidget(
                                  keyboardType: ITextInputType.text,
                                  maxLines: 1,
                                  textStyle: RechargeConstantText.text16BlackBold,
                                  inputText: cardInfo.stateName ?? '',
                                  hintText: S.of(context).strCardState,
                                  hintStyle: RechargeConstantText.text16Hint,
                                  fieldCallBack: (content) {
                                    cardInfo.stateName = content;
                                    cardInfo.billState = content;
                                    setState(() {});
                                  },
                                  contentPadding: EdgeInsets.fromLTRB(
                                      ScreenUtil().setWidth(6),
                                      ScreenUtil().setWidth(15),
                                      ScreenUtil().setWidth(6),
                                      ScreenUtil().setWidth(15)),
                                  focusNode: focusNode4,
                                  cursorColor: RechargeConstantColors.app_color,
                                  inputBorder: const UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFFDDDDDD))),
                                  enabled: cardInfo.billCountry != 'US' &&
                                      cardInfo.billCountry != 'CA',
                                  focusedBorder: UnderlineInputBorder(
                                      borderSide: BorderSide(
                                          color: RechargeConstantColors.app_color)),
                                  textInputAction: TextInputAction.next,
                                  inputMaxLength:
                                      TEXT_NUMBER_NAME),
                              Positioned(
                                left: 0,
                                child:
                                    Text('*', style: RechargeConstantText.text14Blue),
                              ),
                              Visibility(
                                visible: cardInfo.billCountry == 'US' ||
                                    cardInfo.billCountry == 'CA',
                                child: Positioned(
                                  right: ScreenUtil().setWidth(8),
                                  child: Image.asset(
                                      RechargeConstantIcons.card_down,
                                      width: ScreenUtil().setWidth(24),
                                      height: ScreenUtil().setWidth(24)),
                                ),
                              ),
                            ],
                          ),
                        ),
                      ),
                      Visibility(
                        visible: cardCheck &&
                            ((cardInfo.billCountry == 'US' ||
                                        cardInfo.billCountry == 'CA') &&
                                    cardInfo.stateName?.isEmpty != false ||
                                cardInfo.billCountry != 'US' &&
                                    cardInfo.billCountry != 'CA' &&
                                    (cardInfo.stateName?.length ?? 0) < 5),
                        child: Padding(
                          padding: EdgeInsets.fromLTRB(
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8),
                              ScreenUtil().setWidth(15),
                              0),
                          child: Text(
                              cardInfo.billCountry == 'US' ||
                                      cardInfo.billCountry == 'CA'
                                  ? S.of(context).strCardStateCheckTip
                                  : S.of(context).strCardStateTip,
                              style: RechargeConstantText.text12Red),
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.fromLTRB(ScreenUtil().setWidth(15),
                            0, ScreenUtil().setWidth(15), 0),
                        child: Stack(
                          alignment: Alignment.center,
                          clipBehavior: Clip.none,
                          children: [
                            EditTextWidget(
                                keyboardType: ITextInputType.phone,
                                maxLines: 1,
                                textStyle: RechargeConstantText.text16BlackBold,
                                inputText: cardInfo.billZip ?? '',
                                hintText: S.of(context).strCardPost,
                                hintStyle: RechargeConstantText.text16Hint,
                                fieldCallBack: (content) {
                                  cardInfo.billZip = content;
                                  setState(() {});
                                },
                                contentPadding: EdgeInsets.fromLTRB(
                                    ScreenUtil().setWidth(6),
                                    ScreenUtil().setWidth(15),
                                    ScreenUtil().setWidth(6),
                                    ScreenUtil().setWidth(15)),
                                focusNode: focusNode5,
                                cursorColor: RechargeConstantColors.app_color,
                                inputBorder: const UnderlineInputBorder(
                                    borderSide:
                                        BorderSide(color: Color(0xFFDDDDDD))),
                                focusedBorder: UnderlineInputBorder(
                                    borderSide: BorderSide(
                                        color: RechargeConstantColors.app_color)),
                                textInputAction: TextInputAction.next,
                                inputMaxLength: TEXT_NUMBER_NAME),
                            Positioned(
                              left: 0,
                              child: Text('*', style: RechargeConstantText.text14Blue),
                            ),
                          ],
                        ),
                      ),
                      Visibility(
                        visible:
                            cardCheck && (cardInfo.billZip?.length ?? 0) < 2,
                        child: Padding(
                          padding: EdgeInsets.fromLTRB(
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8),
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8)),
                          child: Text(S.of(context).strCardPostTip,
                              style: RechargeConstantText.text12Red),
                        ),
                      ),
                      Container(
                          height: ScreenUtil().setWidth(10),
                          color: RechargeConstantColors.card_line_color),
                      Padding(
                        padding: EdgeInsets.fromLTRB(
                            ScreenUtil().setWidth(15),
                            ScreenUtil().setWidth(24),
                            ScreenUtil().setWidth(15),
                            0),
                        child: Text.rich(
                          TextSpan(
                            text: '*',
                            style: RechargeConstantText.text14Blue,
                            children: <TextSpan>[
                              TextSpan(
                                  text: S.of(context).strCardAddress1,
                                  style: RechargeConstantText.text14Hint2),
                            ],
                          ),
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.fromLTRB(ScreenUtil().setWidth(15),
                            0, ScreenUtil().setWidth(15), 0),
                        child: EditTextWidget(
                            keyboardType: ITextInputType.text,
                            maxLines: 1,
                            textStyle: RechargeConstantText.text16BlackBold,
                            inputText: cardInfo.addressLine1 ?? '',
                            hintText: S.of(context).strCardAddress1Hint,
                            hintStyle: RechargeConstantText.text16Hint,
                            fieldCallBack: (content) {
                              cardInfo.addressLine1 = content;
                              setState(() {});
                            },
                            contentPadding: EdgeInsets.fromLTRB(
                                0,
                                ScreenUtil().setWidth(15),
                                0,
                                ScreenUtil().setWidth(15)),
                            focusNode: focusNode6,
                            cursorColor: RechargeConstantColors.app_color,
                            inputBorder: const UnderlineInputBorder(
                                borderSide:
                                    BorderSide(color: Color(0xFFDDDDDD))),
                            focusedBorder: UnderlineInputBorder(
                                borderSide: BorderSide(
                                    color: RechargeConstantColors.app_color)),
                            textInputAction: TextInputAction.next,
                            inputMaxLength: TEXT_SIZE_ACCOUNT),
                      ),
                      Visibility(
                        visible: cardCheck &&
                            (cardInfo.addressLine1?.length ?? 0) < 5,
                        child: Padding(
                          padding: EdgeInsets.fromLTRB(
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8),
                              ScreenUtil().setWidth(15),
                              ScreenUtil().setWidth(8)),
                          child: Text(S.of(context).strCardAddress1Tip,
                              style: RechargeConstantText.text12Red),
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.fromLTRB(
                            ScreenUtil().setWidth(15),
                            ScreenUtil().setWidth(16),
                            ScreenUtil().setWidth(15),
                            0),
                        child: Text(S.of(context).strCardAddress2,
                            style: RechargeConstantText.text14Hint2),
                      ),
                      Padding(
                        padding: EdgeInsets.fromLTRB(ScreenUtil().setWidth(15),
                            0, ScreenUtil().setWidth(15), 0),
                        child: EditTextWidget(
                            keyboardType: ITextInputType.text,
                            maxLines: 1,
                            textStyle: RechargeConstantText.text16BlackBold,
                            inputText: cardInfo.addressLine2 ?? '',
                            hintText: S.of(context).strCardAddress2Tip,
                            hintStyle: RechargeConstantText.text16Hint,
                            fieldCallBack: (content) {
                              cardInfo.addressLine2 = content;
                              setState(() {});
                            },
                            contentPadding: EdgeInsets.fromLTRB(
                                0,
                                ScreenUtil().setWidth(15),
                                0,
                                ScreenUtil().setWidth(15)),
                            focusNode: focusNode7,
                            cursorColor: RechargeConstantColors.app_color,
                            inputBorder: const UnderlineInputBorder(
                                borderSide:
                                    BorderSide(color: Color(0xFFDDDDDD))),
                            focusedBorder: UnderlineInputBorder(
                                borderSide: BorderSide(
                                    color: RechargeConstantColors.app_color)),
                            inputMaxLength: TEXT_SIZE_ACCOUNT),
                      ),
                      Container(
                          height: ScreenUtil().setWidth(10),
                          color: RechargeConstantColors.card_line_color),
                    ],
                  ),
                ),
              ),
              Divider(
                  color: RechargeConstantColors.line_color,
                  height: ScreenUtil().setWidth(1)),
              RechargeKeyBoardHeightWidget(
                child: Align(
                  alignment: Alignment.bottomCenter,
                  child: Padding(
                    padding: EdgeInsets.fromLTRB(
                        ScreenUtil().setWidth(15),
                        ScreenUtil().setWidth(10),
                        ScreenUtil().setWidth(15),
                        ScreenUtil().setWidth(20)),
                    child: ShakeAnimatedWidget(
                      duration: const Duration(milliseconds: 200),
                      enabled: cardAnim,
                      shakeAngle: Rotation.deg(x: 4, y: 4, z: 1),
                      child: ButtonWidget(
                        S.of(context).strCardAddressSave,
                        RechargeConstantText.text20WhiteBold,
                        color: Colors.black,
                        shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(4),
                        ),
                        padding: EdgeInsets.all(ScreenUtil().setWidth(13)),
                        onPressed: NavigatorUtils.debounce(() {
                          cardCheckEmail = (cardInfo.billEmail?.length ?? 0) >=
                                  2 &&
                              !RegExp('^[a-zA-Z0-9_.-]+@[a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)*\.[a-zA-Z0-9]{2,6}\$')
                                  .hasMatch(cardInfo.billEmail ?? '');
                          if (cardInfo.countryName?.isEmpty != false ||
                              (cardInfo.billFirstName?.length ?? 0) < 2 ||
                              (cardInfo.billLastName?.length ?? 0) < 2 ||
                              (cardInfo.phone?.length ?? 0) < 5 ||
                              (cardInfo.billEmail?.length ?? 0) < 5 ||
                              (cardInfo.billCity?.length ?? 0) < 2 ||
                              ((cardInfo.billCountry == 'US' ||
                                          cardInfo.billCountry == 'CA') &&
                                      cardInfo.stateName?.isEmpty != false ||
                                  cardInfo.billCountry != 'US' &&
                                      cardInfo.billCountry != 'CA' &&
                                      (cardInfo.stateName?.length ?? 0) < 5) ||
                              (cardInfo.billZip?.length ?? 0) < 2 ||
                              (cardInfo.addressLine1?.length ?? 0) < 5 ||
                              cardCheckEmail) {
                            RechargeMediaUtil.instance.startVibration(repeat: -1);
                            cardAnim = true;
                            cardCheck = true;
                            setState(() {});
                            timer?.cancel();
                            timer = Timer(Duration(seconds: 1), () {
                              if (!mounted) return;
                              cardAnim = false;
                              setState(() {});
                            });
                          } else {
                            cardInfo.billAddress =
                                '${cardInfo.addressLine1} ${cardInfo.addressLine2 ?? ''}';
                            widget.logEvent?.call('关闭保存信用卡信息');
                            Navigator.pop(context, true);
                          }
                        }),
                      ),
                    ),
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  Widget _buildDialogItem(Country country) {
    return Row(children: <Widget>[
      CountryPickerUtils.getDefaultFlagImage(country),
      SizedBox(width: ScreenUtil().setWidth(8)),
      Text('+${country.phoneCode}', style: RechargeConstantText.text14White),
      SizedBox(width: ScreenUtil().setWidth(8)),
      Flexible(child: Text(country.name, style: RechargeConstantText.text16WhiteBold))
    ]);
  }

  void showStateDialog(BuildContext context) {
    FocusScope.of(context).unfocus();
    int position = stateIndex;
    List<CardStateList> commitMaps =
        cardInfo.billCountry == 'US' ? usList : caList;
    if (commitMaps.isEmpty) {
      return;
    }
    NavigatorUtils.showBaseDialog(
        context: context,
        dismiss: false,
        name: 'showCommitOptionDialog',
        builder: (BuildContext showContext) {
          return StatefulBuilder(builder: (showContext, showState) {
            return Center(
              child: Container(
                width: ScreenUtil().screenWidth - ScreenUtil().setWidth(60),
                decoration: BoxDecoration(
                    color: Colors.white,
                    borderRadius: BorderRadius.circular(4)),
                padding: EdgeInsets.only(top: ScreenUtil().setWidth(5)),
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Stack(
                      children: [
                        Container(
                          width: ScreenUtil().screenWidth -
                              ScreenUtil().setWidth(60),
                          padding: EdgeInsets.only(
                              top: ScreenUtil().setWidth(15),
                              bottom: ScreenUtil().setWidth(15)),
                          child: Text(
                            S.of(context).strCardState,
                            style: RechargeConstantText.text20BlackBold,
                            textAlign: TextAlign.center,
                          ),
                        ),
                        Positioned(
                          top: ScreenUtil().setWidth(5),
                          right: ScreenUtil().setWidth(5),
                          child: IconButton(
                            padding: EdgeInsets.all(0),
                            icon: Image.asset(
                              RechargeConstantIcons.app_pop_close,
                              width: ScreenUtil().setWidth(40),
                              height: ScreenUtil().setWidth(40),
                            ),
                            onPressed: NavigatorUtils.debounce(() {
                              Navigator.pop(context);
                            }),
                          ),
                        ),
                      ],
                    ),
                    Container(
                      constraints:
                          BoxConstraints(maxHeight: ScreenUtil().setWidth(300)),
                      child: ListView.builder(
                        physics: BouncingScrollPhysics(),
                        shrinkWrap: true,
                        itemCount: commitMaps.length,
                        itemBuilder: (BuildContext itemContext, int index) {
                          return ButtonWidget(
                            commitMaps[index].name,
                            position == index
                                ? RechargeConstantText.text16Black
                                : RechargeConstantText.text16Hint,
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            iconData: Image.asset(
                                position == index
                                    ? RechargeConstantIcons.pop_selected
                                    : RechargeConstantIcons.pop_unselected,
                                width: ScreenUtil().setWidth(16),
                                height: ScreenUtil().setWidth(16)),
                            color: Colors.transparent,
                            right: true,
                            textExpanded: true,
                            padding: EdgeInsets.all(ScreenUtil().setWidth(15)),
                            onPressed: NavigatorUtils.debounce(() {
                              position = index;
                              showState(() {});
                            }),
                          );
                        },
                      ),
                    ),
                    Padding(
                      padding: EdgeInsets.fromLTRB(
                          ScreenUtil().setWidth(20),
                          ScreenUtil().setWidth(10),
                          ScreenUtil().setWidth(20),
                          ScreenUtil().setWidth(20)),
                      child: ButtonWidget(
                        S.of(context).strAppOk,
                        RechargeConstantText.text20WhiteBold,
                        color: Colors.black,
                        shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(4),
                        ),
                        padding: EdgeInsets.all(ScreenUtil().setWidth(13)),
                        onPressed: NavigatorUtils.debounce(() {
                          Navigator.pop(context);
                        }),
                      ),
                    ),
                  ],
                ),
              ),
            );
          });
        }).then((value) {
      stateIndex = position;
      cardInfo.billState = commitMaps[stateIndex].code;
      cardInfo.stateName = commitMaps[stateIndex].name;
      setState(() {});
    });
  }
}
