import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:animated_widgets/widgets/rotation_animated.dart';
import 'package:animated_widgets/widgets/shake_animated_widget.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:mmoo_base/entity/card_entity.dart';
import 'package:mmoo_base/provider/user_provider.dart';
import 'package:mmoo_base/ui/button_widget.dart';
import 'package:mmoo_base/ui/edit_text_widget.dart';
import 'package:mmoo_base/utils/navigator_utils.dart';
import 'package:mmoo_base_recharge/card/address_page.dart';
import 'package:mmoo_base_recharge/card/utils/common_utils.dart';
import 'package:mmoo_base_recharge/card/utils/keyboard_height_widget.dart';
import 'package:mmoo_base_recharge/card/utils/media_util.dart';
import 'package:mmoo_base_recharge/card/utils/recharge_constant.dart';
import 'package:mmoo_base_recharge/generated/l10n.dart';
import 'package:mmoo_base_recharge/recharge_utils.dart';
import 'package:provider/provider.dart';
import 'package:sp_util/sp_util.dart';

//card
class CardPage extends StatefulWidget {
  final String? rechargeId;
  final String? rechargePrice;
  final String? rechargeCoins;
  final logEvent;

  const CardPage(this.rechargeId, this.rechargePrice, this.rechargeCoins,
      {Key? key, this.logEvent})
      : super(key: key);

  @override
  _CardPageState createState() => _CardPageState();
}

class _CardPageState extends State<CardPage> {
  String CARD_INFO = 'card-info';

  late CardEntity cardInfo;
  String cardDate = '';

  // bool cardDone = false;
  bool cardSave = false;
  bool cardAddressSave = false;
  FocusNode focusNode = FocusNode();
  FocusNode focusNode1 = FocusNode();
  FocusNode focusNode2 = FocusNode();
  bool cardAnim = false;
  bool cardCheck = false;
  Timer? timer;

  @override
  void initState() {
    super.initState();
    SystemChrome.setSystemUIOverlayStyle(SystemUiOverlayStyle(
        systemNavigationBarColor: const Color(0xFF000000),
        systemNavigationBarDividerColor: null,
        statusBarColor: Colors.transparent,
        systemNavigationBarIconBrightness:
            Platform.isAndroid ? Brightness.dark : Brightness.light,
        statusBarIconBrightness:
            Platform.isAndroid ? Brightness.dark : Brightness.light,
        statusBarBrightness:
            Platform.isAndroid ? Brightness.dark : Brightness.light));
    try {
      String cardStr = SpUtil.getString(CARD_INFO, defValue: '{}')!;
      cardInfo = CardEntity.fromJson(jsonDecode(cardStr));
    } catch (e) {
      cardInfo = CardEntity();
    }
    cardInfo.orderAmount = widget.rechargePrice ?? '0';
    cardAddressSave = cardInfo.billFirstName?.isEmpty == false &&
        cardInfo.billLastName?.isEmpty == false &&
        cardInfo.addressLine1?.isEmpty == false &&
        cardInfo.billCity?.isEmpty == false &&
        cardInfo.countryName?.isEmpty == false;
    // cardDone = cardInfo.cardNo?.isEmpty == false &&
    //     cardInfo.cardName?.isEmpty == false &&
    //     cardInfo.cardSecurityCode?.isEmpty == false &&
    //     cardInfo.cardExpireMonth?.isEmpty == false &&
    //     cardInfo.year?.isEmpty == false;
    // cardSave = cardAddressSave && cardDone;
    cardSave = cardAddressSave;
    widget.logEvent?.call('点击信用卡支付');
  }

  @override
  void dispose() {
    SystemChrome.setSystemUIOverlayStyle(SystemUiOverlayStyle(
        systemNavigationBarColor: const Color(0xFF000000),
        systemNavigationBarDividerColor: null,
        statusBarColor: Colors.transparent,
        systemNavigationBarIconBrightness:
            Platform.isAndroid ? Brightness.light : Brightness.dark,
        statusBarIconBrightness:
            Platform.isAndroid ? Brightness.light : Brightness.dark,
        statusBarBrightness:
            Platform.isAndroid ? Brightness.light : Brightness.dark));
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Consumer<UserProvider>(builder: (_, UserProvider userProvider, __) {
      return WillPopScope(
        onWillPop: () {
          showExitDialog();
          return Future.value(false);
        },
        child: GestureDetector(
          behavior: HitTestBehavior.opaque,
          onTap: () {
            focusNode.unfocus();
            focusNode1.unfocus();
            focusNode2.unfocus();
          },
          child: Scaffold(
            resizeToAvoidBottomInset: false,
            backgroundColor: Colors.white,
            body: SafeArea(
              maintainBottomViewPadding: true,
              child: Column(
                children: <Widget>[
                  RechargeCommonUtils.getAppBarWidget(
                      context, S.of(context).strCardPayment, black: true,
                      pop: () {
                    showExitDialog();
                  }),
                  Divider(
                      color: RechargeConstantColors.line_color,
                      height: ScreenUtil().setWidth(1)),
                  Expanded(
                    child: SingleChildScrollView(
                      physics: const BouncingScrollPhysics(),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Padding(
                              padding: EdgeInsets.only(
                                  left: ScreenUtil().setWidth(15),
                                  top: ScreenUtil().setWidth(15)),
                              child: Text(S.of(context).strCardSupport,
                                  style: RechargeConstantText.text14Hint2)),
                          Padding(
                            padding: EdgeInsets.fromLTRB(
                                ScreenUtil().setWidth(15),
                                ScreenUtil().setWidth(10),
                                ScreenUtil().setWidth(15),
                                ScreenUtil().setWidth(17)),
                            child: Row(
                              children: [
                                Image.asset(RechargeConstantIcons.card_VISA,
                                    width: ScreenUtil().setWidth(40),
                                    height: ScreenUtil().setWidth(30)),
                                SizedBox(width: ScreenUtil().setWidth(15)),
                                Image.asset(
                                    RechargeConstantIcons.card_mastercard,
                                    width: ScreenUtil().setWidth(40),
                                    height: ScreenUtil().setWidth(30)),
                                SizedBox(width: ScreenUtil().setWidth(15)),
                                Image.asset(RechargeConstantIcons.card_americon,
                                    width: ScreenUtil().setWidth(40),
                                    height: ScreenUtil().setWidth(30)),
                                SizedBox(width: ScreenUtil().setWidth(15)),
                                Image.asset(
                                    RechargeConstantIcons.card_dinersclub,
                                    width: ScreenUtil().setWidth(40),
                                    height: ScreenUtil().setWidth(30)),
                                SizedBox(width: ScreenUtil().setWidth(15)),
                                Image.asset(RechargeConstantIcons.card_discover,
                                    width: ScreenUtil().setWidth(40),
                                    height: ScreenUtil().setWidth(30)),
                              ],
                            ),
                          ),
                          Container(
                              height: ScreenUtil().setWidth(10),
                              color: RechargeConstantColors.card_line_color),
                          Padding(
                            padding: EdgeInsets.fromLTRB(
                                ScreenUtil().setWidth(15),
                                ScreenUtil().setWidth(24),
                                ScreenUtil().setWidth(15),
                                0),
                            child: Text.rich(
                              TextSpan(
                                text: '*',
                                style: RechargeConstantText.text14Blue,
                                children: <TextSpan>[
                                  TextSpan(
                                      text: S.of(context).strCardNumber,
                                      style: RechargeConstantText.text14Hint2),
                                ],
                              ),
                            ),
                          ),
                          Padding(
                            padding: EdgeInsets.fromLTRB(
                                ScreenUtil().setWidth(15),
                                0,
                                ScreenUtil().setWidth(15),
                                0),
                            child: Stack(
                              alignment: Alignment.centerRight,
                              children: [
                                EditTextWidget(
                                    keyboardType: ITextInputType.cardNo,
                                    maxLines: 1,
                                    textStyle:
                                        RechargeConstantText.text20BlackBold,
                                    inputText: cardInfo.cardNo ?? '',
                                    hintText: S.of(context).strCardNumberHint,
                                    hintStyle:
                                        RechargeConstantText.text20HintBold,
                                    fieldCallBack: (content) {
                                      if ((cardInfo.cardNo?.length ?? 0) <
                                              content.length &&
                                          (content.length == 4 ||
                                              content.length == 10 ||
                                              content.length == 16) &&
                                          !content.endsWith('  ')) {
                                        content += '  ';
                                        cardInfo.cardNo = content;
                                        setState(() {});
                                      } else if ((cardInfo.cardNo?.length ??
                                                  0) <
                                              content.length &&
                                          (content.length == 5 ||
                                              content.length == 11 ||
                                              content.length == 17) &&
                                          !content.endsWith('  ')) {
                                        content = content.substring(
                                                0, content.length - 1) +
                                            '  ' +
                                            content.substring(
                                                content.length - 1,
                                                content.length);
                                        cardInfo.cardNo = content;
                                        setState(() {});
                                      } else if ((cardInfo.cardNo?.length ??
                                                  0) <
                                              content.length &&
                                          (content.length == 6 ||
                                              content.length == 12 ||
                                              content.length == 18) &&
                                          !content.endsWith(' ')) {
                                        content = content.substring(
                                                0, content.length - 1) +
                                            ' ' +
                                            content.substring(
                                                content.length - 1,
                                                content.length);
                                        cardInfo.cardNo = content;
                                        setState(() {});
                                      } else {
                                        cardInfo.cardNo = content;
                                        if (cardCheck) {
                                          setState(() {});
                                        }
                                      }
                                    },
                                    contentPadding: EdgeInsets.fromLTRB(
                                        ScreenUtil().setWidth(6),
                                        ScreenUtil().setWidth(15),
                                        ScreenUtil().setWidth(6),
                                        ScreenUtil().setWidth(15)),
                                    focusNode: focusNode,
                                    cursorColor:
                                        RechargeConstantColors.app_color,
                                    inputBorder: const UnderlineInputBorder(
                                        borderSide: BorderSide(
                                            color: Color(0xFFDDDDDD))),
                                    focusedBorder: const UnderlineInputBorder(
                                        borderSide: BorderSide(
                                            color: RechargeConstantColors
                                                .app_color)),
                                    textInputAction: TextInputAction.next,
                                    inputMaxLength: 22),
                                Image.asset(RechargeConstantIcons.card_card,
                                    width: ScreenUtil().setWidth(40),
                                    height: ScreenUtil().setWidth(40)),
                              ],
                            ),
                          ),
                          Visibility(
                            visible: cardCheck &&
                                (cardInfo.cardNo?.length ?? 0) < 22,
                            child: Padding(
                              padding: EdgeInsets.fromLTRB(
                                  ScreenUtil().setWidth(15),
                                  ScreenUtil().setWidth(8),
                                  ScreenUtil().setWidth(15),
                                  ScreenUtil().setWidth(8)),
                              child: Text(S.of(context).strCardNumberTip,
                                  style: RechargeConstantText.text12Red),
                            ),
                          ),
                          Padding(
                            padding: EdgeInsets.fromLTRB(
                                ScreenUtil().setWidth(15),
                                ScreenUtil().setWidth(24),
                                ScreenUtil().setWidth(15),
                                0),
                            child: Row(
                              children: [
                                Expanded(
                                  child: Text.rich(
                                    TextSpan(
                                      text: '*',
                                      style: RechargeConstantText.text14Blue,
                                      children: <TextSpan>[
                                        TextSpan(
                                            text: S.of(context).strCardDate,
                                            style: RechargeConstantText
                                                .text14Hint2),
                                      ],
                                    ),
                                  ),
                                ),
                                Expanded(
                                  child: Text.rich(
                                    TextSpan(
                                      text: '*',
                                      style: RechargeConstantText.text14Blue,
                                      children: <TextSpan>[
                                        TextSpan(
                                            text: S.of(context).strCardCVV,
                                            style: RechargeConstantText
                                                .text14Hint2),
                                      ],
                                    ),
                                  ),
                                ),
                              ],
                            ),
                          ),
                          Padding(
                            padding: EdgeInsets.fromLTRB(
                                ScreenUtil().setWidth(15),
                                0,
                                ScreenUtil().setWidth(15),
                                0),
                            child: Row(
                              children: [
                                Expanded(
                                  child: EditTextWidget(
                                      keyboardType: ITextInputType.cardDate,
                                      maxLines: 1,
                                      textStyle:
                                          RechargeConstantText.text20BlackBold,
                                      inputText: cardDate.isEmpty == false ||
                                              cardInfo.year?.isEmpty != false
                                          ? cardDate
                                          : '${cardInfo.cardExpireMonth ?? ''}/${cardInfo.year ?? ''}',
                                      hintText: S.of(context).strCardDateHint,
                                      hintStyle:
                                          RechargeConstantText.text20HintBold,
                                      fieldCallBack: (content) {
                                        if (cardDate.length < content.length &&
                                            content.length == 2 &&
                                            !content.endsWith('/')) {
                                          cardInfo.cardExpireMonth = content;
                                          content += '/';
                                          cardDate = content;
                                          setState(() {});
                                          return;
                                        } else if (cardDate.length <
                                                content.length &&
                                            content.length == 3 &&
                                            !content.endsWith('/')) {
                                          content = content.substring(
                                                  0, content.length - 1) +
                                              '/' +
                                              content.substring(
                                                  content.length - 1,
                                                  content.length);
                                          cardDate = content;
                                          setState(() {});
                                          return;
                                        } else if (content.length == 5) {
                                          cardInfo.year =
                                              content.substring(3, 5);
                                        } else if (content.isEmpty) {
                                          cardInfo.cardExpireMonth = '';
                                          cardInfo.year = '';
                                        } else {
                                          cardInfo.year = '';
                                        }
                                        cardDate = content;
                                        if (cardCheck) {
                                          setState(() {});
                                        }
                                      },
                                      contentPadding: EdgeInsets.fromLTRB(
                                          ScreenUtil().setWidth(6),
                                          ScreenUtil().setWidth(15),
                                          ScreenUtil().setWidth(6),
                                          ScreenUtil().setWidth(15)),
                                      focusNode: focusNode1,
                                      cursorColor:
                                          RechargeConstantColors.app_color,
                                      inputBorder: const UnderlineInputBorder(
                                          borderSide: BorderSide(
                                              color: Color(0xFFDDDDDD))),
                                      focusedBorder: const UnderlineInputBorder(
                                          borderSide: BorderSide(
                                              color: RechargeConstantColors
                                                  .app_color)),
                                      textInputAction: TextInputAction.next,
                                      inputMaxLength: 5),
                                ),
                                Container(
                                    color:
                                        RechargeConstantColors.card_line_color,
                                    height: ScreenUtil().setWidth(40),
                                    width: ScreenUtil().setWidth(1)),
                                Expanded(
                                  child: Stack(
                                    alignment: Alignment.centerRight,
                                    children: [
                                      EditTextWidget(
                                          keyboardType: ITextInputType.number,
                                          maxLines: 1,
                                          textStyle: RechargeConstantText
                                              .text20BlackBold,
                                          inputText: cardInfo
                                                  .cardSecurityCode ??
                                              '',
                                          hintText: S
                                              .of(context)
                                              .strCardCVVHint,
                                          hintStyle: RechargeConstantText
                                              .text20HintBold,
                                          fieldCallBack: (content) {
                                            cardInfo.cardSecurityCode = content;
                                            if (cardCheck) {
                                              setState(() {});
                                            }
                                          },
                                          contentPadding: EdgeInsets.fromLTRB(
                                              ScreenUtil().setWidth(6),
                                              ScreenUtil().setWidth(15),
                                              ScreenUtil().setWidth(6),
                                              ScreenUtil().setWidth(15)),
                                          focusNode: focusNode2,
                                          cursorColor: RechargeConstantColors
                                              .app_color,
                                          inputBorder:
                                              const UnderlineInputBorder(
                                                  borderSide: BorderSide(
                                                      color:
                                                          Color(0xFFDDDDDD))),
                                          focusedBorder:
                                              const UnderlineInputBorder(
                                                  borderSide: BorderSide(
                                                      color:
                                                          RechargeConstantColors
                                                              .app_color)),
                                          textInputAction: TextInputAction.next,
                                          inputMaxLength: 4),
                                      IconButton(
                                        padding: const EdgeInsets.all(0),
                                        iconSize: ScreenUtil().setWidth(24),
                                        icon: Image.asset(
                                          RechargeConstantIcons.card_tip,
                                          width: ScreenUtil().setWidth(24),
                                          height: ScreenUtil().setWidth(24),
                                        ),
                                        onPressed: NavigatorUtils.debounce(() {
                                          showCardCVVDialog(context);
                                        }),
                                      ),
                                    ],
                                  ),
                                ),
                              ],
                            ),
                          ),
                          Padding(
                            padding: EdgeInsets.fromLTRB(
                                ScreenUtil().setWidth(15),
                                ScreenUtil().setWidth(8),
                                ScreenUtil().setWidth(15),
                                ScreenUtil().setWidth(8)),
                            child: Row(
                              children: [
                                Expanded(
                                  child: Visibility(
                                    visible: cardCheck &&
                                        cardInfo.year?.isEmpty != false,
                                    child: Text(S.of(context).strCardDateTip,
                                        style: RechargeConstantText.text12Red),
                                  ),
                                ),
                                Expanded(
                                  child: Visibility(
                                    visible: cardCheck &&
                                        (cardInfo.cardSecurityCode?.length ??
                                                0) <
                                            3,
                                    child: Text(S.of(context).strCardCVVTip,
                                        style: RechargeConstantText.text12Red),
                                  ),
                                ),
                              ],
                            ),
                          ),
                          // CreditCardInputForm(
                          //   cardHeight: ScreenUtil().setWidth(200),
                          //   cardNumber: cardInfo.cardNo ?? '',
                          //   cardName: cardInfo.cardName ?? '',
                          //   cardCVV: cardInfo.cardSecurityCode ?? '',
                          //   cardValid: cardInfo.cardExpireMonth?.isEmpty ==
                          //               false &&
                          //           cardInfo.year?.isEmpty == false
                          //       ? '${cardInfo.cardExpireMonth ?? ''}/${cardInfo.year ?? ''}'
                          //       : '',
                          //   showResetButton: true,
                          //   initialAutoFocus: false,
                          //   intialCardState:
                          //       cardDone ? InputState.done : InputState.number,
                          //   onStateChange: (currentState, CardInfo info) {
                          //     if (currentState == InputState.done) {
                          //       cardInfo.cardNo = info.cardNumber;
                          //       cardInfo.cardName = info.name;
                          //       cardInfo.cardSecurityCode = info.cvv;
                          //       List<String>? validates =
                          //           info.validate?.split('/');
                          //       if (validates?.isEmpty == false) {
                          //         cardInfo.cardExpireMonth = validates?[0];
                          //       } else {
                          //         cardInfo.cardExpireMonth = '';
                          //       }
                          //       if ((validates?.length ?? 0) > 1) {
                          //         cardInfo.year = validates?[1];
                          //       } else {
                          //         cardInfo.year = '';
                          //       }
                          //       cardDone = cardInfo.cardNo?.isEmpty == false &&
                          //           cardInfo.cardName?.isEmpty == false &&
                          //           cardInfo.cardSecurityCode?.isEmpty ==
                          //               false &&
                          //           cardInfo.cardExpireMonth?.isEmpty ==
                          //               false &&
                          //           cardInfo.year?.isEmpty == false;
                          //     } else {
                          //       cardDone = false;
                          //     }
                          //   },
                          //   frontCardDecoration: BoxDecoration(
                          //     borderRadius: BorderRadius.circular(30),
                          //     gradient: const LinearGradient(
                          //       colors: [
                          //         ConstantColors.app_color,
                          //         ConstantColors.appColorLeft,
                          //       ],
                          //     ),
                          //   ),
                          //   backCardDecoration: BoxDecoration(
                          //     borderRadius: BorderRadius.circular(30),
                          //     gradient: const LinearGradient(
                          //       colors: [
                          //         ConstantColors.app_color,
                          //         ConstantColors.appColorLeft,
                          //       ],
                          //     ),
                          //   ),
                          //   prevButtonDecoration: BoxDecoration(
                          //     borderRadius: BorderRadius.circular(30),
                          //     gradient: const LinearGradient(
                          //       colors: [
                          //         ConstantColors.app_color,
                          //         ConstantColors.appColorLeft,
                          //       ],
                          //     ),
                          //   ),
                          //   nextButtonDecoration: BoxDecoration(
                          //     borderRadius: BorderRadius.circular(30),
                          //     gradient: const LinearGradient(
                          //       colors: [
                          //         ConstantColors.app_color,
                          //         ConstantColors.appColorLeft,
                          //       ],
                          //     ),
                          //   ),
                          //   resetButtonDecoration: BoxDecoration(
                          //     borderRadius: BorderRadius.circular(30),
                          //     gradient: const LinearGradient(
                          //       colors: [
                          //         ConstantColors.app_color,
                          //         ConstantColors.appColorLeft,
                          //       ],
                          //     ),
                          //   ),
                          //   prevButtonTextStyle: ConstantText.text16WhiteBold,
                          //   nextButtonTextStyle: ConstantText.text16WhiteBold,
                          //   resetButtonTextStyle: ConstantText.text16WhiteBold,
                          // ),
                          // SizedBox(height: ScreenUtil().setWidth(15)),
                          // Divider(
                          //     color: ConstantColors.line_color,
                          //     indent: ScreenUtil().setWidth(15),
                          //     endIndent: ScreenUtil().setWidth(15),
                          //     height: ScreenUtil().setWidth(1)),
                          Padding(
                            padding: EdgeInsets.fromLTRB(
                                ScreenUtil().setWidth(15),
                                ScreenUtil().setWidth(10),
                                ScreenUtil().setWidth(15),
                                ScreenUtil().setWidth(10)),
                            child: Row(
                              mainAxisAlignment: MainAxisAlignment.spaceBetween,
                              children: [
                                Text(S.of(context).strCardRemember,
                                    style: RechargeConstantText.text14Hint2),
                                CupertinoSwitch(
                                  value: cardSave,
                                  onChanged: (open) {
                                    setState(() {
                                      cardSave = open;
                                    });
                                  },
                                ),
                              ],
                            ),
                          ),
                          Container(
                              height: ScreenUtil().setWidth(10),
                              color: RechargeConstantColors.card_line_color),
                          Padding(
                            padding: EdgeInsets.fromLTRB(
                                ScreenUtil().setWidth(15),
                                ScreenUtil().setWidth(12),
                                ScreenUtil().setWidth(15),
                                0),
                            child: Text.rich(
                              TextSpan(
                                text: '*',
                                style: RechargeConstantText.text14Blue,
                                children: <TextSpan>[
                                  TextSpan(
                                      text: S.of(context).strCardBilling,
                                      style: RechargeConstantText.text14Hint2),
                                ],
                              ),
                            ),
                          ),
                          InkWell(
                            child: Padding(
                              padding: EdgeInsets.fromLTRB(
                                  ScreenUtil().setWidth(15),
                                  ScreenUtil().setWidth(24),
                                  ScreenUtil().setWidth(15),
                                  ScreenUtil().setWidth(24)),
                              child: Row(
                                children: [
                                  Expanded(
                                    child: cardAddressSave &&
                                            cardInfo.billFirstName?.isEmpty ==
                                                false &&
                                            cardInfo.billLastName?.isEmpty ==
                                                false &&
                                            cardInfo.addressLine1?.isEmpty ==
                                                false &&
                                            cardInfo.billCity?.isEmpty ==
                                                false &&
                                            cardInfo.countryName?.isEmpty ==
                                                false
                                        ? Column(
                                            crossAxisAlignment:
                                                CrossAxisAlignment.start,
                                            children: [
                                              Text(
                                                  '${cardInfo.billFirstName} ${cardInfo.billLastName}',
                                                  style: RechargeConstantText
                                                      .text18BlackBold),
                                              SizedBox(
                                                  height:
                                                      ScreenUtil().setWidth(5)),
                                              Text(cardInfo.billAddress ?? '',
                                                  style: RechargeConstantText
                                                      .text16Hint2),
                                              SizedBox(
                                                  height:
                                                      ScreenUtil().setWidth(5)),
                                              Text(
                                                  '${cardInfo.billCity} ${cardInfo.countryName}',
                                                  style: RechargeConstantText
                                                      .text16Hint2),
                                            ],
                                          )
                                        : Row(
                                            children: [
                                              Image.asset(
                                                  RechargeConstantIcons
                                                      .card_address,
                                                  width:
                                                      ScreenUtil().setWidth(40),
                                                  height: ScreenUtil()
                                                      .setWidth(40)),
                                              Expanded(
                                                child: Text(
                                                    S
                                                        .of(context)
                                                        .strCardBillingTip,
                                                    style: RechargeConstantText
                                                        .text16Hint),
                                              ),
                                            ],
                                          ),
                                  ),
                                  Image.asset(RechargeConstantIcons.card_right,
                                      width: ScreenUtil().setWidth(40),
                                      height: ScreenUtil().setWidth(40)),
                                ],
                              ),
                            ),
                            onTap: NavigatorUtils.debounce(() {
                              widget.logEvent?.call('点击填写信用卡账单地址');
                              NavigatorUtils.navigatorPush(
                                      context,
                                      AddressPage(cardInfo,
                                          logEvent: widget.logEvent),
                                      'AddressPage')
                                  .then((value) {
                                if (value == true) {
                                  cardAddressSave = true;
                                  setState(() {});
                                }
                              });
                            }),
                          ),
                          Visibility(
                            visible: cardCheck && !cardAddressSave,
                            child: Padding(
                              padding: EdgeInsets.fromLTRB(
                                  ScreenUtil().setWidth(15),
                                  0,
                                  ScreenUtil().setWidth(15),
                                  ScreenUtil().setWidth(8)),
                              child: Text(S.of(context).strCardBillingToast,
                                  style: RechargeConstantText.text12Red),
                            ),
                          ),
                          Container(
                              height: ScreenUtil().setWidth(150),
                              color: RechargeConstantColors.card_line_color),
                        ],
                      ),
                    ),
                  ),
                  Divider(
                      color: RechargeConstantColors.line_color,
                      height: ScreenUtil().setWidth(1)),
                  RechargeKeyBoardHeightWidget(
                    child: Align(
                      alignment: Alignment.bottomCenter,
                      child: SingleChildScrollView(
                        physics: const BouncingScrollPhysics(),
                        child: Column(
                          mainAxisSize: MainAxisSize.min,
                          mainAxisAlignment: MainAxisAlignment.end,
                          children: [
                            Padding(
                              padding: EdgeInsets.fromLTRB(
                                  ScreenUtil().setWidth(15),
                                  ScreenUtil().setWidth(12),
                                  ScreenUtil().setWidth(15),
                                  ScreenUtil().setWidth(12)),
                              child: Row(
                                mainAxisAlignment:
                                    MainAxisAlignment.spaceBetween,
                                children: [
                                  Text(S.of(context).strCardGold,
                                      style: RechargeConstantText.text14Hint2),
                                  Expanded(
                                    child: Text(
                                      S.of(context).strAnchorCoins(
                                          widget.rechargeCoins ?? '0'),
                                      style:
                                          RechargeConstantText.text14BlackBold,
                                      textAlign: TextAlign.right,
                                    ),
                                  ),
                                ],
                              ),
                            ),
                            Padding(
                              padding: EdgeInsets.fromLTRB(
                                  ScreenUtil().setWidth(15),
                                  ScreenUtil().setWidth(12),
                                  ScreenUtil().setWidth(15),
                                  ScreenUtil().setWidth(12)),
                              child: Row(
                                mainAxisAlignment:
                                    MainAxisAlignment.spaceBetween,
                                children: [
                                  Text(S.of(context).strCardAmount,
                                      style: RechargeConstantText.text14Hint2),
                                  Expanded(
                                    child: Text(
                                      cardInfo.orderAmount ?? '',
                                      style:
                                          RechargeConstantText.text18BlueBold,
                                      textAlign: TextAlign.right,
                                    ),
                                  ),
                                ],
                              ),
                            ),
                            Padding(
                              padding: EdgeInsets.fromLTRB(
                                  ScreenUtil().setWidth(15),
                                  ScreenUtil().setWidth(10),
                                  ScreenUtil().setWidth(15),
                                  ScreenUtil().setWidth(20)),
                              child: ShakeAnimatedWidget(
                                duration: const Duration(milliseconds: 200),
                                enabled: cardAnim,
                                shakeAngle: Rotation.deg(x: 4, y: 4, z: 1),
                                child: ButtonWidget(
                                  S.of(context).strCardPurchase,
                                  RechargeConstantText.text20WhiteBold,
                                  color: Colors.black,
                                  shape: RoundedRectangleBorder(
                                    borderRadius: BorderRadius.circular(4),
                                  ),
                                  padding:
                                      EdgeInsets.all(ScreenUtil().setWidth(13)),
                                  onPressed: NavigatorUtils.debounce(() {
                                    // if (!cardDone) {
                                    //   MooCommonUtils.showCustomToast(
                                    //       context, S.of(context).strCardToast);
                                    // } else
                                    if (!cardAddressSave ||
                                        (cardInfo.cardNo?.length ?? 0) < 22 ||
                                        cardInfo.year?.isEmpty != false ||
                                        (cardInfo.cardSecurityCode?.length ??
                                                0) <
                                            3) {
                                      widget.logEvent?.call('提交信用卡支付_资料未完善');
                                      RechargeMediaUtil.instance
                                          .startVibration(repeat: -1);
                                      cardCheck = true;
                                      cardAnim = true;
                                      setState(() {});
                                      timer?.cancel();
                                      timer =
                                          Timer(const Duration(seconds: 1), () {
                                        if (!mounted) return;
                                        cardAnim = false;
                                        setState(() {});
                                      });
                                    } else {
                                      widget.logEvent?.call('提交信用卡支付');
                                      if (cardSave) {
                                        SpUtil.putString(
                                            CARD_INFO, jsonEncode(cardInfo));
                                      } else {
                                        SpUtil.putString(CARD_INFO, '{}');
                                      }
                                      cardInfo.orderCurrency = 'USD';
                                      cardInfo.cardExpireYear =
                                          '20${cardInfo.year}';
                                      cardInfo.billPhone =
                                          '+${cardInfo.phoneCode} ${cardInfo.phone}';
                                      RechargeUtils.instance.connectionTBF(
                                          widget.rechargeId,
                                          cardInfo.orderAmount,
                                          cardInfo, back: (value) {
                                        if (value == true) {
                                          Navigator.pop(context);
                                        }
                                      }).then((type) {
                                        if (type != -2) {
                                          checkPpyPurchase(type);
                                        }
                                      });
                                    }
                                  }),
                                ),
                              ),
                            ),
                          ],
                        ),
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ),
        ),
      );
    });
  }

  Future showCardCVVDialog(BuildContext context) {
    FocusScope.of(context).unfocus();
    return NavigatorUtils.showBaseDialog(
        context: context,
        name: 'showCardCVVDialog',
        builder: (BuildContext showContext) {
          return Center(
            child: Container(
              width: ScreenUtil().screenWidth - ScreenUtil().setWidth(60),
              decoration: BoxDecoration(
                color: Colors.white,
                borderRadius: BorderRadius.circular(4),
              ),
              padding: EdgeInsets.all(ScreenUtil().setWidth(20)),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Text(
                    S.of(context).strCardCVVWhat,
                    style: RechargeConstantText.text18BlackBold,
                    textAlign: TextAlign.center,
                  ),
                  SizedBox(height: ScreenUtil().setWidth(20)),
                  Text(
                    S.of(context).strCardCVVWhatTip,
                    style: RechargeConstantText.text14Black,
                    textAlign: TextAlign.center,
                  ),
                  SizedBox(height: ScreenUtil().setWidth(15)),
                  Row(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Expanded(
                        child: Column(
                          children: [
                            Image.asset(RechargeConstantIcons.card_CVV3,
                                width: ScreenUtil().setWidth(112),
                                height: ScreenUtil().setWidth(70)),
                            Text(
                              S.of(context).strCardCVVWhat3,
                              style: RechargeConstantText.text12Hint,
                              textAlign: TextAlign.center,
                            ),
                          ],
                        ),
                      ),
                      Expanded(
                        child: Column(
                          children: [
                            Image.asset(RechargeConstantIcons.card_CVV4,
                                width: ScreenUtil().setWidth(112),
                                height: ScreenUtil().setWidth(70)),
                            Text(
                              S.of(context).strCardCVVWhat4,
                              style: RechargeConstantText.text12Hint,
                              textAlign: TextAlign.center,
                            ),
                          ],
                        ),
                      ),
                    ],
                  ),
                  SizedBox(height: ScreenUtil().setWidth(20)),
                  ButtonWidget(
                    S.of(context).strAppOk,
                    RechargeConstantText.text20WhiteBold,
                    color: Colors.black,
                    shape: RoundedRectangleBorder(
                      borderRadius: BorderRadius.circular(4),
                    ),
                    padding: EdgeInsets.all(ScreenUtil().setWidth(13)),
                    onPressed: NavigatorUtils.debounce(() {
                      Navigator.pop(context);
                    }),
                  ),
                  SizedBox(height: ScreenUtil().setWidth(10)),
                ],
              ),
            ),
          );
        });
  }

  void showExitDialog() {
    RechargeCommonUtils.showCustomDialog(context,
        content: S.of(context).strCardExit,
        right: S.of(context).strLoginAboutExit,
        debounce: false, callback: (position) {
      if (position == 1) {
        Navigator.pop(context);
        Navigator.pop(context);
      } else if (position == 0) {
        Navigator.pop(context);
      }
    });
  }

//return 0接口失败 -1取消 1支付中 2加币成功 3重复加币
  void checkPpyPurchase(int type) {
    RechargeUtils.instance
        .checkPpyPurchase(RechargeUtils.instance.orderId, show: true, type: 1)
        .then((value) {
      if (value == -1) {
        widget.logEvent?.call('信用卡取消支付');
        RechargeCommonUtils.showCustomToast(
            context, S.of(context).strCardCancel);
      } else if (value == 0) {
        widget.logEvent?.call('信用卡支付失败');
        showCardTypeDialog(context, type);
      } else if (value == 2) {
        widget.logEvent?.call('信用卡支付成功_成功加币');
        Navigator.pop(context, true);
      } else if (value == 3) {
        widget.logEvent?.call('信用卡支付成功_重复加币');
        RechargeCommonUtils.showCustomToast(
            context, S.of(context).strCardSuccess);
        Navigator.pop(context, true);
      }
    });
  }

  //0支付失败 -1支付中 1支付成功等待回调
  Future showCardTypeDialog(BuildContext context, int type) {
    FocusScope.of(context).unfocus();
    return NavigatorUtils.showBaseDialog(
        context: context,
        name: 'showCardErrorDialog',
        builder: (BuildContext showContext) {
          return Center(
            child: Container(
              width: ScreenUtil().screenWidth - ScreenUtil().setWidth(60),
              decoration: BoxDecoration(
                color: Colors.white,
                borderRadius: BorderRadius.circular(4),
              ),
              padding: EdgeInsets.all(ScreenUtil().setWidth(20)),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Image.asset(
                      type == 0
                          ? RechargeConstantIcons.card_error
                          : RechargeConstantIcons.pop_success,
                      width: ScreenUtil().setWidth(100),
                      height: ScreenUtil().setWidth(100)),
                  SizedBox(height: ScreenUtil().setWidth(20)),
                  Text(
                    type == 0
                        ? S.of(context).strCardError
                        : S.of(context).strCardSuccessTip,
                    style: RechargeConstantText.text14Black,
                    textAlign: TextAlign.center,
                  ),
                  SizedBox(height: ScreenUtil().setWidth(20)),
                  ButtonWidget(
                    type == 0
                        ? S.of(context).strCardErrorChange
                        : S.of(context).strCardSuccessNow,
                    RechargeConstantText.text20WhiteBold,
                    color: Colors.black,
                    shape: RoundedRectangleBorder(
                      borderRadius: BorderRadius.circular(4),
                    ),
                    padding: EdgeInsets.all(ScreenUtil().setWidth(13)),
                    onPressed: NavigatorUtils.debounce(() {
                      if (type == 0) {
                        Navigator.pop(context);
                      } else {
                        Navigator.pop(context);
                        checkPpyPurchase(type);
                      }
                    }),
                  ),
                  SizedBox(height: ScreenUtil().setWidth(10)),
                  ButtonWidget(
                    type == 0
                        ? S.of(context).strCardErrorSwitch
                        : S.of(context).strCardSuccessLate,
                    RechargeConstantText.text20BlackBold,
                    color: Colors.white,
                    shape: RoundedRectangleBorder(
                      side: BorderSide(
                        color: Colors.black,
                        width: ScreenUtil().setWidth(2),
                      ),
                      borderRadius: BorderRadius.circular(4),
                    ),
                    padding: EdgeInsets.all(ScreenUtil().setWidth(13)),
                    onPressed: NavigatorUtils.debounce(() {
                      Navigator.pop(context);
                      Navigator.pop(context);
                    }),
                  ),
                ],
              ),
            ),
          );
        }).then((value) {
      if (value == true || (type == 2 || type == 3)) {
        Navigator.pop(context);
      }
    });
  }
}
