import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:mmoo_base/config/app_config.dart';
import 'package:mmoo_base/ui/button_widget.dart';
import 'package:mmoo_base/utils/navigator_utils.dart';
import 'package:mmoo_base_recharge/card/utils/recharge_constant.dart';
import 'package:mmoo_base_recharge/generated/l10n.dart';

///通用逻辑-基本方法管理
class RechargeCommonUtils {
  static void showCustomToast(BuildContext context, String msg) {
    AppConfig.printLog('showCustomToast: $msg');
    try {
      FToast fToast = FToast();
      fToast.init(context);
      fToast.showToast(
          child: Container(
            padding: EdgeInsets.all(ScreenUtil().setWidth(30)),
            decoration: BoxDecoration(
              color: RechargeConstantColors.bg_pop_color,
              borderRadius: BorderRadius.circular(20),
            ),
            child: Text(
              msg,
              style: RechargeConstantText.text16WhiteBold,
              textAlign: TextAlign.center,
            ),
          ),
          gravity: ToastGravity.CENTER,
          toastDuration: Duration(seconds: 2));
    } catch (e) {
      AppConfig.printLog(e);
    }
  }

  ///导航栏
  static Widget getAppBarWidget(BuildContext context, String title,
      {Widget? rightChild,
      bool back = true,
      bool top = false,
      bool black = false,
      Function? pop}) {
    return Container(
      height: kToolbarHeight,
      margin: EdgeInsets.only(
        top: top ? ScreenUtil().statusBarHeight : 0,
      ),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          back
              ? IconButton(
                  padding: EdgeInsets.all(0),
                  icon: Image.asset(
                    black
                        ? RechargeConstantIcons.card_close
                        : RechargeConstantIcons.app_return,
                    width: ScreenUtil().setWidth(40),
                    height: ScreenUtil().setWidth(40),
                  ),
                  onPressed: NavigatorUtils.debounce(() {
                    if (pop == null) {
                      Navigator.pop(context);
                    } else {
                      pop();
                    }
                  }),
                )
              : SizedBox(width: ScreenUtil().setWidth(20)),
          Expanded(
            child: Text(title,
                style: black
                    ? RechargeConstantText.text20BlackBold
                    : RechargeConstantText.text20WhiteBold,
                maxLines: 1,
                overflow: TextOverflow.ellipsis),
          ),
          rightChild ?? Container()
        ],
      ),
    );
  }

  static void showCustomDialog(BuildContext context,
      {String? title,
        String? content,
        String? left,
        String? right,
        bool showLeft = true,
        bool debounce = true,
        bool barrierDismissible = true,
        bool highLeft = true,
        callback}) {
    FocusScope.of(context).unfocus();
    NavigatorUtils.showBaseDialog(
        context: context,
        dismiss: barrierDismissible,
        name: 'showCustomDialog',
        builder: (BuildContext showContext) {
          return Center(
            child: Container(
              width: ScreenUtil().screenWidth - ScreenUtil().setWidth(80),
              decoration: BoxDecoration(
                color: RechargeConstantColors.bg_pop_color,
                borderRadius: BorderRadius.circular(20),
              ),
              child: Stack(
                children: [
                  Padding(
                    padding: EdgeInsets.fromLTRB(
                      ScreenUtil().setWidth(20),
                      ScreenUtil().setWidth(20),
                      ScreenUtil().setWidth(20),
                      ScreenUtil().setWidth(30),
                    ),
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      children: <Widget>[
                        Text(title ?? S.of(context).strAppTip,
                            style: RechargeConstantText.text20WhiteBold,
                            textAlign: TextAlign.center),
                        SizedBox(height: ScreenUtil().setWidth(20)),
                        Text(content ?? '',
                            style: RechargeConstantText.text16White,
                            textAlign: TextAlign.center),
                        SizedBox(height: ScreenUtil().setWidth(30)),
                        showLeft
                            ? Row(
                          children: <Widget>[
                            Expanded(
                              child: ButtonWidget(
                                  left ?? S.of(context).strAppLater,
                                  highLeft
                                      ? RechargeConstantText.text14WhiteBold
                                      : RechargeConstantText.text14White60Bold,
                                  color: highLeft
                                      ? RechargeConstantColors.app_color
                                      : RechargeConstantColors.color_btn_un,
                                  shape: StadiumBorder(),
                                  padding: EdgeInsets.only(
                                      top: ScreenUtil().setWidth(12),
                                      bottom: ScreenUtil().setWidth(12)),
                                  onPressed: debounce
                                      ? NavigatorUtils.debounce(() {
                                    Navigator.pop(context, true);
                                    callback?.call(0);
                                  })
                                      : () => callback?.call(0)),
                            ),
                            SizedBox(width: ScreenUtil().setWidth(20)),
                            Expanded(
                              child: ButtonWidget(
                                  right ?? S.of(context).strAppSure,
                                  highLeft
                                      ? RechargeConstantText.text14White60Bold
                                      : RechargeConstantText.text14WhiteBold,
                                  color: highLeft
                                      ? RechargeConstantColors.color_btn_un
                                      : RechargeConstantColors.app_color,
                                  shape: StadiumBorder(),
                                  padding: EdgeInsets.only(
                                      top: ScreenUtil().setWidth(12),
                                      bottom: ScreenUtil().setWidth(12)),
                                  onPressed: debounce
                                      ? NavigatorUtils.debounce(() {
                                    Navigator.pop(context, true);
                                    callback?.call(1);
                                  })
                                      : () => callback?.call(1)),
                            ),
                          ],
                        )
                            : ButtonWidget(
                          right ?? S.of(context).strAppSure,
                          RechargeConstantText.text14WhiteBold,
                          color: RechargeConstantColors.app_color,
                          shape: StadiumBorder(),
                          padding: EdgeInsets.only(
                              top: ScreenUtil().setWidth(12),
                              bottom: ScreenUtil().setWidth(12)),
                          mainAxisSize: MainAxisSize.max,
                          onPressed: debounce
                              ? NavigatorUtils.debounce(() {
                            Navigator.pop(context, true);
                            callback?.call(1);
                          })
                              : () => callback?.call(1),
                        ),
                      ],
                    ),
                  ),
                  Positioned(
                    top: ScreenUtil().setWidth(12),
                    right: ScreenUtil().setWidth(3),
                    child: IconButton(
                      padding: EdgeInsets.all(0),
                      icon: Image.asset(
                        RechargeConstantIcons.app_pop_close,
                        width: ScreenUtil().setWidth(40),
                        height: ScreenUtil().setWidth(40),
                      ),
                      onPressed: NavigatorUtils.debounce(() {
                        Navigator.pop(context, true);
                        callback?.call(-1);
                      }),
                    ),
                  ),
                ],
              ),
            ),
          );
        }).then((value) {
      if (value == null) {
        callback?.call(2);
      }
    });
  }
}
